@@ -162,15 +162,13 @@ namespace MonoDevelop.CSharp.Formatting
 			if (textPolicy != null) {
 				data.Options.TabsToSpaces = textPolicy.TabsToSpaces;
 				data.Options.TabSize = textPolicy.TabWidth;
-				data.Options.DefaultEolMarker = textPolicy.GetEolMarker ();
 			}
-			data.Options.OverrideDocumentEolMarker = true;
 			data.Text = input;
 
-//			System.Console.WriteLine ("-----");
-//			System.Console.WriteLine (data.Text.Replace (" ", ".").Replace ("\t", "->"));
-//			System.Console.WriteLine ("-----");
-
+			// System.Console.WriteLine ("-----");
+			// System.Console.WriteLine (data.Text.Replace (" ", ".").Replace ("\t", "->"));
+			// System.Console.WriteLine ("-----");
+			
 			var parser = new CSharpParser ();
 			var compilationUnit = parser.Parse (data);
 			bool hadErrors = parser.HasErrors;
@@ -181,18 +179,18 @@ namespace MonoDevelop.CSharp.Formatting
 				return input.Substring (startOffset, Math.Max (0, Math.Min (endOffset, input.Length) - startOffset));
 			}
 			var adapter = new TextEditorDataAdapter (data);
-			var formattingVisitor = new ICSharpCode.NRefactory.CSharp.AstFormattingVisitor (policy.CreateOptions (), adapter, new FormattingActionFactory (data)) {
+			var factory = new FormattingActionFactory (data);
+			var formattingVisitor = new ICSharpCode.NRefactory.CSharp.AstFormattingVisitor (policy.CreateOptions (), adapter, factory) {
 				HadErrors = hadErrors
 			};
 			
 			compilationUnit.AcceptVisitor (formattingVisitor, null);
 			
-			
 			var changes = new List<ICSharpCode.NRefactory.CSharp.Refactoring.Action> ();
-
 			changes.AddRange (formattingVisitor.Changes.
 				Where (c => (startOffset <= c.Offset && c.Offset < endOffset)));
-			
+			var endPositionChange = factory.CreateTextReplaceAction (endOffset, 0, null);
+			changes.Add (endPositionChange);
 			MDRefactoringContext.MdScript.RunActions (changes, null);
 			
 			// check if the formatter has produced errors
@@ -200,21 +198,13 @@ namespace MonoDevelop.CSharp.Formatting
 			parser.Parse (data);
 			if (parser.HasErrors) {
 				LoggingService.LogError ("C# formatter produced source code errors. See console for output.");
-				Console.WriteLine (data.Text);
 				return input.Substring (startOffset, Math.Max (0, Math.Min (endOffset, input.Length) - startOffset));
 			}
-				
-			int end = endOffset;
-			foreach (TextReplaceAction c in changes) {
-				end -= c.RemovedChars;
-				if (c.InsertedText != null)
-					end += c.InsertedText.Length;
-			}
 			
-		/*			System.Console.WriteLine ("-----");
+/*			System.Console.WriteLine ("-----");
 			System.Console.WriteLine (data.Text.Replace (" ", "^").Replace ("\t", "->"));
 			System.Console.WriteLine ("-----");*/
-			string result = data.GetTextBetween (startOffset, Math.Min (data.Length, end));
+			string result = data.GetTextBetween (startOffset, Math.Min (data.Length, endPositionChange.Offset));
 			data.Dispose ();
 			return result;
 		}
