@@ -27,20 +27,21 @@
 //
 
 using System;
-using System.Linq;
 using System.Text;
-
+using System.Linq;
 using Mono.Cecil;
 
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Dom;
 using MonoDevelop.Projects.Dom.Output;
-using MonoDevelop.Ide.Gui;
-using MonoDevelop.Ide.Gui.Pads;
 using MonoDevelop.Ide.Gui.Components;
 using Mono.TextEditor.Highlighting;
-using MonoDevelop.Components.Commands;
+using MonoDevelop.Ide;
+using ICSharpCode.Decompiler.Ast;
+using ICSharpCode.Decompiler;
+using System.Threading;
+using Mono.TextEditor;
+using System.Collections.Generic;
 
 namespace MonoDevelop.AssemblyBrowser
 {
@@ -95,14 +96,14 @@ namespace MonoDevelop.AssemblyBrowser
 			};
 			DomTypeNodeBuilder.settings.EmitNameCallback = delegate (INode domVisitable, ref string outString) {
 				if (domVisitable is IType) {
-					outString = "<span style=\"text.link\"><u><a ref=\"T:" + ((IType)domVisitable).FullName + "\">" + outString + "</a></u></span>";
+					outString = "<span style=\"text.link\"><u><a ref=\"" + ((IType)domVisitable).HelpUrl + "\">" + outString + "</a></u></span>";
 				} else {
 					outString = "<span style=\"text\">" + outString + "</span>";
 				}
 			};
 			DomTypeNodeBuilder.settings.PostProcessCallback = delegate (INode domVisitable, ref string outString) {
 				if (domVisitable is IReturnType) {
-					outString = "<span style=\"text.link\"><u><a ref=\"T:" + ((IReturnType)domVisitable).FullName + "\">" + outString + "</a></u></span>";
+					outString = "<span style=\"text.link\"><u><a ref=\"" + ((IReturnType)domVisitable).HelpUrl + "\">" + outString + "</a></u></span>";
 				}
 			};
 		}
@@ -127,7 +128,7 @@ namespace MonoDevelop.AssemblyBrowser
 			IType type = (IType)dataObject;
 			ctx.AddChild (new BaseTypeFolder (type));
 			bool publicOnly = ctx.Options ["PublicApiOnly"];
-			ctx.AddChildren (type.Members.Where (member => !(member.IsSpecialName && !(member is IMethod && ((IMethod)member).IsConstructor)) && !(publicOnly && !member.IsPublic)));
+			ctx.AddChildren (type.Members.Where (member => !(member.IsSpecialName && !(member is IMethod && ((IMethod)member).IsConstructor)) && !(publicOnly && !(member.IsPublic || member.IsProtected))));
 		}
 		
 		public override bool HasChildNodes (ITreeBuilder builder, object dataObject)
@@ -163,138 +164,16 @@ namespace MonoDevelop.AssemblyBrowser
 			return result.ToString ();
 		}
 		
-		public string GetDisassembly (ITreeNavigator navigator)
+		public List<ReferenceSegment> Disassemble (TextEditorData data, ITreeNavigator navigator)
 		{
-			IType type = (IType)navigator.DataItem;
-			StringBuilder result = new StringBuilder ();
-			result.Append (DomMethodNodeBuilder.GetAttributes (Ambience, type.Attributes));
-			result.Append (Ambience.GetString (type, settings));
-			bool first = true;
-			
-			if (type.ClassType == ClassType.Enum) {
-				result.Append ("<span style=\"text\"> {</span>");
-				result.Append ("");
-				result.AppendLine ();
-				int length = result.Length;
-				foreach (IField field in type.Fields) {
-					if ((field.Modifiers & Modifiers.SpecialName) == Modifiers.SpecialName)
-						continue;
-					result.Append ("<span style=\"text\"> \t");
-					result.Append (field.Name);
-					length = result.Length;
-					result.Append (",</span>");
-					result.AppendLine ();
-				}
-				result.Length = length;
-				result.AppendLine ();
-				result.Append ("<span style=\"text\">}</span>");
-				return result.ToString ();
-			}
-			result.AppendLine ();
-			result.Append ("<span style=\"text\">{</span>");
-			
-//			Style colorStyle = TextEditorOptions.Options.GetColorStyle (widget);
-//			ChunkStyle comments = colorStyle.GetChunkStyle ("comment");
-//			string commentSpan = String.Format ("<span foreground=\"#{0:X6}\">", comments.Color.Pixel);
-			string commentSpan = "<span style=\"comment\">";
-			foreach (IField field in type.Fields) {
-				if ((field.Modifiers & Modifiers.SpecialName) == Modifiers.SpecialName)
-					continue;
-				if (first) {
-					result.AppendLine ();
-					result.Append ("\t");
-					result.Append (commentSpan);
-					result.Append (Ambience.SingleLineComment (GettextCatalog.GetString ("Fields")));
-					result.Append ("</span>");
-					result.AppendLine ();
-				}
-				first = false;
-				result.Append ("\t");
-				result.Append (Ambience.GetString (field, settings));
-				result.Append ("<span style=\"text\">;</span>");
-				result.AppendLine ();
-			}
-			first = true;
-			foreach (IEvent evt in type.Events) {
-				if (first) {
-					result.AppendLine ();
-					result.Append ("\t");
-					result.Append (commentSpan);
-					result.Append (Ambience.SingleLineComment (GettextCatalog.GetString ("Events")));
-					result.Append ("</span>");
-					result.AppendLine ();
-				}
-				first = false;
-				result.Append ("\t");
-				result.Append (Ambience.GetString (evt, settings));
-				result.Append ("<span style=\"text\">;</span>");
-				result.AppendLine ();
-			}
-			first = true;
-			foreach (IMethod method in type.Methods) {
-				if (!method.IsConstructor)
-					continue;
-				if (first) {
-					result.AppendLine ();
-					result.Append ("\t");
-					result.Append (commentSpan);
-					result.Append (Ambience.SingleLineComment (GettextCatalog.GetString ("Constructors")));
-					result.Append ("</span>");
-					result.AppendLine ();
-				}
-				first = false;
-				result.Append ("\t");
-				result.Append (Ambience.GetString (method, settings));
-				result.Append ("<span style=\"text\">;</span>");
-				result.AppendLine ();
-			}
-			first = true;
-			foreach (IMethod method in type.Methods) {
-				if ((method.Modifiers & Modifiers.SpecialName) == Modifiers.SpecialName || method.IsConstructor)
-					continue;
-				if (first) {
-					result.AppendLine ();
-					result.Append ("\t");
-					result.Append (commentSpan);
-					result.Append (Ambience.SingleLineComment (GettextCatalog.GetString ("Methods")));
-					result.Append ("</span>");
-					result.AppendLine ();
-				}
-				first = false;
-				result.Append ("\t");
-				result.Append (Ambience.GetString (method, settings));
-				result.Append ("<span style=\"text\">;</span>");
-				result.AppendLine ();
-			}
-			first = true;
-			foreach (IProperty property in type.Properties) {
-				if (first) {
-					result.AppendLine ();
-					result.Append ("\t");
-					result.Append (commentSpan);
-					result.Append (Ambience.SingleLineComment (GettextCatalog.GetString ("Properties")));
-					result.Append ("</span>");
-					result.AppendLine ();
-				}
-				first = false;
-				result.Append ("\t");
-				result.Append (Ambience.GetString (property, settings));
-				result.Append (" <span style=\"text\">{</span>");
-				if (property.HasGet)
-					result.Append (" <span style=\"keyword.property\">get</span><span style=\"text\">;</span>");
-				if (property.HasSet)
-					result.Append (" <span style=\"keyword.property\">set</span><span style=\"text\">;</span>");
-				result.Append (" <span style=\"text\">}</span>");
-				result.AppendLine ();
-			}
-			result.Append ("<span style=\"text\">}</span>");
-			
-			result.AppendLine ();
-			return result.ToString ();
+			var type = (DomCecilType)navigator.DataItem;
+			return DomMethodNodeBuilder.Disassemble (data, rd => rd.DisassembleType (type.TypeDefinition));
 		}
-		public string GetDecompiledCode (ITreeNavigator navigator)
+		
+		public List<ReferenceSegment> Decompile (TextEditorData data, ITreeNavigator navigator)
 		{
-			return GetDisassembly (navigator);
+			var type = (DomCecilType)navigator.DataItem;
+			return DomMethodNodeBuilder.Decompile (data, DomMethodNodeBuilder.GetModule (navigator), type.TypeDefinition, b => b.AddType (type.TypeDefinition));
 		}
 		
 		string IAssemblyBrowserNodeBuilder.GetDocumentationMarkup (ITreeNavigator navigator)
