@@ -32,6 +32,7 @@ using System.Collections.ObjectModel;
 using Gtk;
 using Pango;
 using Gdk;
+using Mono.TextEditor;
 
 namespace MonoDevelop.DesignerSupport.Toolbox
 {
@@ -87,7 +88,6 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				headerLayout.FontDescription = desc;
 		}
 		
-		const int spacing = 2;
 		const int categoryHeaderSize = 25;
 		Pango.Layout layout;
 		Pango.Layout headerLayout;
@@ -132,8 +132,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			categories.Add (category);
 			foreach (Item item in category.Items) {
-				this.iconSize.Width  = Math.Max (this.iconSize.Width,  item.Icon.Width + spacing);
-				this.iconSize.Height  = Math.Max (this.iconSize.Height,  item.Icon.Height + spacing);
+				this.iconSize.Width  = Math.Max (this.iconSize.Width,  item.Icon.Width);
+				this.iconSize.Height  = Math.Max (this.iconSize.Height,  item.Icon.Height);
 			}
 		}
 		
@@ -204,24 +204,24 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 					messageLayout.SetText (CustomMessage);
 				else
 					messageLayout.SetText (MonoDevelop.Core.GettextCatalog.GetString ("There are no tools available for the current document."));
-				draw.DrawLayout (Style.TextGC (StateType.Normal), Allocation.Width * 1 / 6 , 12, messageLayout);
+				draw.DrawLayout (Style.TextGC (StateType.Normal), Allocation.Width * 1 / 6, 12, messageLayout);
 				messageLayout.Dispose ();
 				return true;
 			}
 			
 			Cairo.Context cr = Gdk.CairoHelper.Create (e.Window);
 			draw.DrawRectangle (Style.BaseGC (StateType.Normal), true, area);
-			int xpos = spacing - (this.hAdjustement != null ? (int)this.hAdjustement.Value : 0);
+			int xpos = (this.hAdjustement != null ? (int)this.hAdjustement.Value : 0);
 			int vadjustment = (this.vAdjustement != null ? (int)this.vAdjustement.Value : 0);
-			int ypos = spacing - vadjustment;
+			int ypos = -vadjustment;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 				const int foldSegmentHeight = 8;
 				
 				if (category == SelectedItem) {
 					draw.DrawRectangle (Style.BaseGC (StateType.Selected), 
 					                   true, 
-					                   new Gdk.Rectangle (xpos,
-					                                      ypos,
+					                   new Gdk.Rectangle (xpos, 
+					                                      ypos, 
 					                                      itemDimension.Width, 
 					                                      itemDimension.Height));
 				} else {
@@ -232,16 +232,16 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 					cr.RelLineTo (-itemDimension.Width, 0);
 					cr.RelLineTo (0, -itemDimension.Height);
 					cr.ClosePath ();
-					Cairo.Gradient pat = new Cairo.LinearGradient (xpos, ypos, xpos, ypos + itemDimension.Height);
-					Cairo.Color ccol = Convert (Style.Mid (StateType.Normal));
-					pat.AddColorStop (0, ccol);
-					ccol.A = 0.4;
-					pat.AddColorStop (1, ccol);
-					cr.Pattern = pat;
-					cr.FillPreserve ();
-					
+					using (var pat = new Cairo.LinearGradient (xpos, ypos, xpos, ypos + itemDimension.Height)) {
+						Cairo.Color ccol = Convert (Style.Mid (StateType.Normal));
+						ccol.A = 0.2;
+						pat.AddColorStop (0, ccol);
+						ccol.A = 1;
+						pat.AddColorStop (1, ccol);
+						cr.Pattern = pat;
+						cr.FillPreserve ();
+					}
 				}
-				
 				DrawFoldSegment (draw, xpos + 2, ypos + (itemDimension.Height - foldSegmentHeight) / 2, foldSegmentHeight, foldSegmentHeight, category.IsExpanded, category == mouseOverItem && mouseX < xpos + 2 + 12);
 				headerLayout.SetText (category.Text);
 				int width, height;
@@ -267,13 +267,13 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				if (listMode || !curCategory.CanIconizeItems)  {
 					draw.DrawPixbuf (this.Style.ForegroundGC (StateType.Normal), 
 					                item.Icon, 0, 0, 
-					                xpos + spacing, 
+					                xpos + 4, 
 					                ypos + 1 + (itemDimension.Height - item.Icon.Height) / 2, 
 					                item.Icon.Width, item.Icon.Height, Gdk.RgbDither.None, 0, 0);
 					layout.SetText (item.Text);
 					int width, height;
 					layout.GetPixelSize (out width, out height);
-					draw.DrawLayout (Style.TextGC (item != this.SelectedItem ? StateType.Normal : StateType.Selected), xpos + IconSize.Width, ypos + (itemDimension.Height - height) / 2, layout);
+					draw.DrawLayout (Style.TextGC (item != this.SelectedItem ? StateType.Normal : StateType.Selected), xpos + IconSize.Width + 4, ypos + (itemDimension.Height - height) / 2, layout);
 				} else {
 					draw.DrawPixbuf (this.Style.ForegroundGC (StateType.Normal), 
 					                item.Icon, 0, 0, 
@@ -418,12 +418,14 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			return base.OnScrollEvent (evnt);
 		}
 		
+		public Action<Gdk.EventButton> DoPopupMenu { get; set; }
+		
 		protected override bool OnButtonPressEvent (Gdk.EventButton e)
 		{
 			this.GrabFocus ();
 			HideTooltipWindow ();
 			if (this.mouseOverItem is Category) {
-				if (e.Button == 1) {
+				if (!e.TriggersContextMenu () && e.Button == 1) {
 					Category mouseOverCateogry = (Category)this.mouseOverItem;
 					mouseOverCateogry.IsExpanded = !mouseOverCateogry.IsExpanded;
 				}
@@ -433,29 +435,45 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				this.SelectedItem = mouseOverItem;
 				this.QueueDraw ();
 			}
-			if (e.Type == EventType.TwoButtonPress && this.SelectedItem != null) 
+			if (e.TriggersContextMenu ()) {
+				if (DoPopupMenu != null) {
+					DoPopupMenu (null);
+					return true;
+				}
+			} else if (e.Type == EventType.TwoButtonPress && this.SelectedItem != null) { 
 				this.OnActivateSelectedItem (EventArgs.Empty);
+				return true;
+			}
 			return base.OnButtonPressEvent (e);
 		}
 		
+		protected override bool OnPopupMenu ()
+		{
+			if (DoPopupMenu != null) {
+				DoPopupMenu (null);
+				return true;
+			}
+			return base.OnPopupMenu ();
+		}
+		
 		protected override bool OnMotionNotifyEvent (Gdk.EventMotion e)
 		{
-			int xpos = spacing;
-			int ypos = spacing;
+			int xpos = 0;
+			int ypos = 0;
 			HideTooltipWindow ();
 			var oldItem = mouseOverItem;
 			mouseOverItem = null;
 			this.mouseX = (int)e.X + (int)(this.hAdjustement != null ? this.hAdjustement.Value : 0);
 			this.mouseY = (int)e.Y + (int)(this.vAdjustement != null ? this.vAdjustement.Value : 0);
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
-				if (xpos <= mouseX && mouseX <= xpos + itemDimension.Width + spacing  &&
-				    ypos <= mouseY && mouseY <= ypos + itemDimension.Height + spacing) {
+				if (xpos <= mouseX && mouseX <= xpos + itemDimension.Width  &&
+				    ypos <= mouseY && mouseY <= ypos + itemDimension.Height) {
 					mouseOverItem = category;
 					ShowTooltip (mouseOverItem, TipTimer, (int)e.X + 2, (int)e.Y + 16);
 				}
 			}, delegate (Category curCategory, Item item, Gdk.Size itemDimension) {
-				if (xpos <= mouseX && mouseX <= xpos + itemDimension.Width + spacing  &&
-				    ypos <= mouseY && mouseY <= ypos + itemDimension.Height + spacing) {
+				if (xpos <= mouseX && mouseX <= xpos + itemDimension.Width  &&
+				    ypos <= mouseY && mouseY <= ypos + itemDimension.Height) {
 					mouseOverItem = item;
 					ShowTooltip (mouseOverItem, TipTimer, (int)e.X + 2, (int)e.Y + 16);
 				}
@@ -511,7 +529,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		Category GetCategory (Item item)
 		{
 			Category result = null;
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 			}, delegate (Category curCategory, Item innerItem, Gdk.Size itemDimension) {
 				if (innerItem == item) 
@@ -524,7 +542,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Category result = category;
 			Category last = null;
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category curCategory, Gdk.Size itemDimension) {
 				if (last == category) 
 					result = curCategory;
@@ -538,7 +556,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Item result = item;
 			Gdk.Rectangle rect = GetItemExtends (item);
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 			}, delegate (Category curCategory, Item curItem, Gdk.Size itemDimension) {
 				if (xpos > rect.X && ypos == rect.Y && result == item)
@@ -551,7 +569,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Item result = item;
 			Gdk.Rectangle rect = GetItemExtends (item);
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 			}, delegate (Category curCategory, Item curItem, Gdk.Size itemDimension) {
 				if (xpos < rect.X && ypos == rect.Y)
@@ -566,7 +584,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 						
 			Item result = item;
 			Gdk.Rectangle rect = GetItemExtends (item);
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 			}, delegate (Category curCategory, Item curItem, Gdk.Size itemDimension) {
 				if (ypos > rect.Y && xpos == rect.X && result == item && curCategory == itemCategory)
@@ -580,7 +598,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			Category itemCategory = GetCategory (item);
 			Item result = item;
 			Gdk.Rectangle rect = GetItemExtends (item);
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 			}, delegate (Category curCategory, Item curItem, Gdk.Size itemDimension) {
 				if (ypos < rect.Y && xpos == rect.X && curCategory == itemCategory)
@@ -592,7 +610,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		Gdk.Rectangle GetItemExtends (Item item)
 		{
 			Gdk.Rectangle result = new Gdk.Rectangle (0, 0, 0, 0);
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 				if (item == category)
 					result = new Gdk.Rectangle (xpos, ypos, itemDimension.Width, itemDimension.Height);
@@ -607,7 +625,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Item result = currentItem;
 			Item lastItem = null;
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 				if (currentItem == category && lastItem != null)
 					result = lastItem;
@@ -625,7 +643,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 		{
 			Item result = currentItem;
 			Item lastItem = null;
-			int xpos = spacing, ypos = spacing;
+			int xpos = 0, ypos = 0;
 			Iterate (ref xpos, ref ypos, delegate (Category category, Gdk.Size itemDimension) {
 				if (lastItem == currentItem) {
 					result = category;
@@ -652,8 +670,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			Gdk.Rectangle rect = GetItemExtends (this.SelectedItem);
 			if (this.vAdjustement.Value > rect.Top)
 				this.vAdjustement.Value = rect.Top;
-			if (this.vAdjustement.Value + this.Allocation.Height - spacing < rect.Bottom)
-				this.vAdjustement.Value = rect.Bottom - this.Allocation.Height + spacing;
+			if (this.vAdjustement.Value + this.Allocation.Height < rect.Bottom)
+				this.vAdjustement.Value = rect.Bottom - this.Allocation.Height;
 		}
 		
 		protected override void OnSetScrollAdjustments (Adjustment hAdjustement, Adjustment vAdjustement)
@@ -682,11 +700,11 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 					layout.SetText (item.Text);
 					int x, y;
 					layout.GetPixelSize (out x, out y);
-					y = Math.Max (IconSize.Height, y) + spacing;
+					y = Math.Max (IconSize.Height, y);
 					
-					xpos = spacing;
+					xpos = 0;
 					if (action != null)
-						action (category, item, new Gdk.Size (Allocation.Width - spacing * 2, y));
+						action (category, item, new Gdk.Size (Allocation.Width, y));
 					
 					ypos += y;
 				}
@@ -695,15 +713,15 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			foreach (Item item in category.Items) {
 				if (!item.IsVisible)
 					continue;
-				if (xpos + IconSize.Width + spacing >= this.Allocation.Width) {
-					xpos = spacing;
+				if (xpos + IconSize.Width >= this.Allocation.Width) {
+					xpos = 0;
 					ypos += IconSize.Height;
 				}
 				if (action != null)
 					action (category, item, IconSize);
 				xpos += IconSize.Width;
 			}
-			ypos += IconSize.Height + spacing;
+			ypos += IconSize.Height;
 		}
 		
 		void Iterate (ref int xpos, ref int ypos, CategoryAction catAction, ItemAction action)
@@ -711,7 +729,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			foreach (Category category in this.categories) {
 				if (!category.IsVisible)
 					continue;
-				xpos = spacing;
+				xpos = 0;
 				if (this.showCategories) {
 					
 					layout.SetText (category.Text);
@@ -721,7 +739,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 					
 					
 					if (catAction != null)
-						catAction (category, new Size (this.Allocation.Width - spacing * 2, y));
+						catAction (category, new Size (this.Allocation.Width, y));
 					ypos += y;
 				} 
 				if (category.IsExpanded || !this.showCategories) {
@@ -742,8 +760,8 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 				req.Height = 0;
 				return;
 			}
-			int xpos = spacing;
-			int ypos = spacing;
+			int xpos = 0;
+			int ypos = 0;
 			Iterate (ref xpos, ref ypos, null, null);
 			req.Width  = 50; 
 			req.Height = ypos;
@@ -798,7 +816,7 @@ namespace MonoDevelop.DesignerSupport.Toolbox
 			tooltipWindow.ParentWindow = this.GdkWindow;
 			int ox, oy;
 			this.GdkWindow.GetOrigin (out ox, out oy);
-			tooltipWindow.Move (Math.Max (spacing, ox + tipX), oy + tipY);
+			tooltipWindow.Move (Math.Max (0, ox + tipX), oy + tipY);
 			tooltipWindow.ShowAll ();
 			return false;
 		}
