@@ -277,8 +277,11 @@ namespace Stetic {
 			}
 
 			foreach (WidgetData wd in topLevels) {
-				if (wd.Widget != null)
-					wd.Widget.Destroy ();
+				if (wd.Widget != null) {
+					try {
+						wd.Widget.Destroy ();
+					} catch (Exception e) { Console.WriteLine (e); }
+				}
 			}
 
 			selection = null;
@@ -335,7 +338,7 @@ namespace Stetic {
 					string dir = libElem.GetAttribute ("path");
 					if (dir.Length > 0) {
 						if (basePath != null && !Path.IsPathRooted (dir)) {
-							dir = Path.Combine (basePath, dir);
+							dir = FromOSAgnosticRelPath (Path.Combine (basePath, dir));
 							if (Directory.Exists (dir))
 								dir = Path.GetFullPath (dir);
 						}
@@ -348,7 +351,7 @@ namespace Stetic {
 					string libname = libElem.GetAttribute ("name");
 					if (libname.EndsWith (".dll") || libname.EndsWith (".exe")) {
 						if (basePath != null && !Path.IsPathRooted (libname)) {
-							libname = Path.Combine (basePath, libname);
+							libname = FromOSAgnosticRelPath (Path.Combine (basePath, libname));
 							if (File.Exists (libname))
 								libname = Path.GetFullPath (libname);
 						}
@@ -403,11 +406,22 @@ namespace Stetic {
 			this.fileName = fileName;
 			XmlDocument doc = Write (false);
 			
-			XmlTextWriter writer = null;
+			XmlWriter writer = null;
 			try {
+				//Always normalize line endings to \n so that they remain consistent.
+				//Else, because of the way subdocument are persisted unmodified, we could
+				//end up with mixed line endings.
+				//Also, string arrays are stored in plain text data with \n separators,
+				//and break badly if we have \r\n.
+				var settings = new XmlWriterSettings () {
+					NewLineHandling = NewLineHandling.Replace,
+					Encoding = System.Text.Encoding.UTF8,
+					NewLineChars = "\n",
+					Indent = true,
+				};
+
 				// Write to a temporary file first, just in case something fails
-				writer = new XmlTextWriter (fileName + "~", System.Text.Encoding.UTF8);
-				writer.Formatting = Formatting.Indented;
+				writer = XmlTextWriter.Create (fileName + "~", settings);
 				doc.Save (writer);
 				writer.Close ();
 				
@@ -419,6 +433,18 @@ namespace Stetic {
 			}
 		}
 		
+		//in the cases where a path is relative, we store it using / as a path separator
+		//so the .stetic file doesn't unnecessarily differ between Windows & Linux/Mac
+		string ToOSAgnosticRelPath (string s)
+		{
+			return s.Replace ('\\', '/');
+		}
+
+		string FromOSAgnosticRelPath (string s)
+		{
+			return s.Replace ('/', Path.DirectorySeparatorChar);
+		}
+
 		XmlDocument Write (bool includeUndoInfo)
 		{
 			XmlDocument doc = new XmlDocument ();
@@ -451,7 +477,7 @@ namespace Stetic {
 					foreach (string dir in resolver.Directories) {
 						XmlElement dirElem = doc.CreateElement ("assembly-directory");
 						if (basePath != null)
-							dirElem.SetAttribute ("path", AbsoluteToRelativePath (basePath, dir));
+							dirElem.SetAttribute ("path", ToOSAgnosticRelPath (AbsoluteToRelativePath (basePath, dir)));
 						else
 							dirElem.SetAttribute ("path", dir);
 						toplevel.AppendChild (dirElem);
@@ -463,7 +489,7 @@ namespace Stetic {
 					XmlElement libElem = doc.CreateElement ("widget-library");
 					if (wlib.EndsWith (".dll") || wlib.EndsWith (".exe")) {
 						if (basePath != null)
-							libName = AbsoluteToRelativePath (basePath, wlib);
+							libName = ToOSAgnosticRelPath (AbsoluteToRelativePath (basePath, wlib));
 					}
 
 					libElem.SetAttribute ("name", libName);
@@ -958,7 +984,7 @@ namespace Stetic {
 			m.Popup ();
 		}
 		
-		internal static string AbsoluteToRelativePath (string baseDirectoryPath, string absPath)
+		static string AbsoluteToRelativePath (string baseDirectoryPath, string absPath)
 		{
 			if (!Path.IsPathRooted (absPath))
 				return absPath;
@@ -1079,4 +1105,17 @@ namespace Stetic {
 			}
 		}
 	}
+
+	public static class EncodingUtility
+	{
+		static System.Text.UTF8Encoding utf8NoBom;
+
+		/// This is so we can write XML files on .NET in a compatible way with Mono,
+		/// since .NET's XmlWriter writes the BOM but Mono's does not.
+		public static System.Text.UTF8Encoding UTF8NoBom {
+			get {
+				return utf8NoBom ?? (utf8NoBom = new System.Text.UTF8Encoding (false));
+			}
+		}
+	}
 }
