@@ -249,7 +249,6 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 					continue;
 				if (finf.ProjectFile.BuildAction == BuildAction.Compile) {
 					updateTypes = true;
-					break;
 				} else if (IncludeInSyncedProject (finf.ProjectFile)) {
 					updateProject = true;
 				}
@@ -259,14 +258,17 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 				using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Syncing to Xcode..."))) {
 					//FIXME: make this async (and safely async)
 					//FIXME: only update the project if obj-c types change
-					updateProject = UpdateTypes (monitor, true);
+					updateProject |= UpdateTypes (monitor, true);
 				}
 			}
 			
 			if (updateProject) {
 				using (var monitor = GetStatusMonitor (GettextCatalog.GetString ("Syncing to Xcode..."))) {
 					//FIXME: make this async (and safely async)
+					var running = xcode.CheckRunning ();
 					UpdateXcodeProject (monitor);
+					if (running)
+						xcode.OpenProject ();
 				}
 			}
 		}
@@ -372,7 +374,7 @@ namespace MonoDevelop.MacDev.XcodeSyncing
 					FileService.NotifyFilesChanged (changeCtx.FileSyncJobs.Select (f => f.Original));
 				});
 				
-				if (typesAdded)
+				if (typesAdded && xcode.CheckRunning ())
 					UpdateXcodeProject (monitor);
 				
 				return true;
