@@ -135,26 +135,11 @@ namespace MonoDevelop.VersionControl.Views
 			editor.Document.FoldTreeUpdated += delegate {
 				QueueDraw ();
 			};
-			editor.ButtonPressEvent += OnPopupMenu;
+			editor.DoPopupMenu = ShowPopup;
 			Show ();
 		}
 		
-		void OnPopupMenu (object sender, Gtk.ButtonPressEventArgs args)
-		{
-			if (args.Event.Button == 3) {
-				int textEditorXOffset = (int)args.Event.X - (int)editor.TextViewMargin.XOffset;
-				if (textEditorXOffset < 0)
-					return;
-				this.menuPopupLocation = new Cairo.Point ((int)args.Event.X, (int)args.Event.Y);
-				DocumentLocation loc = editor.PointToLocation (textEditorXOffset, (int)args.Event.Y);
-				if (!editor.IsSomethingSelected || !editor.SelectionRange.Contains (editor.Document.LocationToOffset (loc)))
-					editor.Caret.Location = loc;
-				
-				this.ShowPopup ();
-			}
-		}
-		
-		void ShowPopup ()
+		void ShowPopup (EventButton evt)
 		{
 			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet ("/MonoDevelop/VersionControl/BlameView/ContextMenu");
 			Gtk.Menu menu = IdeApp.CommandService.CreateMenu (cset);
@@ -162,21 +147,12 @@ namespace MonoDevelop.VersionControl.Views
 				this.QueueDraw ();
 			};
 			
-			menu.Popup (null, null, new Gtk.MenuPositionFunc (PositionPopupMenu), 0, Gtk.Global.CurrentEventTime);
-		}
-		
-		Cairo.Point menuPopupLocation;
-		void PositionPopupMenu (Menu menu, out int x, out int y, out bool pushIn)
-		{
-			this.GdkWindow.GetOrigin (out x, out y);
-			x += this.menuPopupLocation.X;
-			y += this.menuPopupLocation.Y;
-			Requisition request = menu.SizeRequest ();
-			Gdk.Rectangle geometry = DesktopService.GetUsableMonitorGeometry (Screen, Screen.GetMonitorAtPoint (x, y));
-			
-			y = Math.Max (geometry.Top, Math.Min (y, geometry.Bottom - request.Height));
-			x = Math.Max (geometry.Left, Math.Min (x, geometry.Right - request.Width));
-			pushIn = true;
+			if (evt != null) {
+				GtkWorkarounds.ShowContextMenu (menu, this, evt);
+			} else {
+				var pt = editor.LocationToPoint (editor.Caret.Location);
+				GtkWorkarounds.ShowContextMenu (menu, editor, new Gdk.Rectangle (pt.X, pt.Y, 1, (int)editor.LineHeight));
+			}
 		}
 		
 		void HandleAdjustmentChanged (object sender, EventArgs e)
@@ -259,25 +235,19 @@ namespace MonoDevelop.VersionControl.Views
 			}
 		}
 		
-		static double GetWheelDelta (Scrollbar scrollbar, ScrollDirection direction)
-		{
-			double delta = System.Math.Pow (scrollbar.Adjustment.PageSize, 2.0 / 3.0);
-			if (direction == ScrollDirection.Up || direction == ScrollDirection.Left)
-				delta = -delta;
-			if (scrollbar.Inverted)
-				delta = -delta;
-			return delta;
-		}
-		
 		protected override bool OnScrollEvent (EventScroll evnt)
 		{
-			Scrollbar scrollWidget = (evnt.Direction == ScrollDirection.Up || evnt.Direction == ScrollDirection.Down) ? (Scrollbar)vScrollBar : hScrollBar;
-			if (scrollWidget.Visible) {
-				double newValue = scrollWidget.Adjustment.Value + GetWheelDelta (scrollWidget, evnt.Direction);
-				newValue = System.Math.Max (System.Math.Min (scrollWidget.Adjustment.Upper  - scrollWidget.Adjustment.PageSize, newValue), scrollWidget.Adjustment.Lower);
-				scrollWidget.Adjustment.Value = newValue;
-			}
-			return base.OnScrollEvent (evnt);
+			var alloc = Allocation;
+			double dx, dy;
+			evnt.GetPageScrollPixelDeltas (alloc.Width, alloc.Height, out dx, out dy);
+			
+			if (dx != 0.0 && hScrollBar.Visible)
+				hAdjustment.AddValueClamped (dx);
+			
+			if (dy != 0.0 && vScrollBar.Visible)
+				vAdjustment.AddValueClamped (dy);
+			
+			return (dx != 0.0 || dy != 0.0) || base.OnScrollEvent (evnt);
 		}
 		
 		protected override void OnSizeRequested (ref Gtk.Requisition requisition)
@@ -467,13 +437,14 @@ namespace MonoDevelop.VersionControl.Views
 			uint grabTime;
 			protected override bool OnButtonPressEvent (EventButton evnt)
 			{
-				if (evnt.Button == 3) {
+				if (evnt.TriggersContextMenu ()) {
 					CommandEntrySet opset = new CommandEntrySet ();
 					opset.AddItem (BlameCommands.ShowDiff);
 					opset.AddItem (BlameCommands.ShowLog);
 					opset.AddItem (Command.Separator);
 					opset.AddItem (BlameCommands.CopyRevision);
-					IdeApp.CommandService.ShowContextMenu (opset, this);
+					IdeApp.CommandService.ShowContextMenu (this, evnt, opset, this);
+					return true;
 				} else {
 					if (evnt.X < leftSpacer) {
 						grabTime = evnt.Time;
