@@ -34,6 +34,8 @@ namespace Mono.Debugging.Client
 		bool allowMethodEvaluation;
 		bool allowToStringCalls;
 		
+		public static readonly char Ellipsis = '…';
+		
 		public static EvaluationOptions DefaultOptions {
 			get {
 				EvaluationOptions ops = new EvaluationOptions ();
@@ -50,6 +52,8 @@ namespace Mono.Debugging.Client
 				ops.UseExternalTypeResolver = true;
 				ops.IntegerDisplayFormat = IntegerDisplayFormat.Decimal;
 				ops.CurrentExceptionTag = "$exception";
+				ops.EllipsizeStrings = true;
+				ops.EllipsizedLength = 100;
 				return ops;
 			}
 		}
@@ -59,6 +63,9 @@ namespace Mono.Debugging.Client
 			return (EvaluationOptions) MemberwiseClone ();
 		}
 		
+		public bool EllipsizeStrings { get; set; }
+		public int EllipsizedLength { get; set; }
+		
 		public int EvaluationTimeout { get; set; }
 		public int MemberEvaluationTimeout { get; set; }
 		public bool AllowTargetInvoke { get; set; }
