@@ -292,7 +292,7 @@ namespace Mono.Debugging.Evaluation
 			string typeName = obj != null ? GetValueTypeName (ctx, obj) : "";
 			
 			if (obj == null || IsNull (ctx, obj)) {
-				return ObjectValue.CreateObject (source, path, GetDisplayTypeName (typeName), "(null)", flags, null);
+				return ObjectValue.CreateNullObject (source, path, GetDisplayTypeName (typeName), flags);
 			}
 			else if (IsPrimitive (ctx, obj) || IsEnum (ctx,obj)) {
 				return ObjectValue.CreatePrimitive (source, path, GetDisplayTypeName (typeName), ctx.Evaluator.TargetObjectToExpression (ctx, obj), flags);
@@ -362,8 +362,7 @@ namespace Mono.Debugging.Evaluation
 			// to avoid problems with objects being invalidated due to evaluations in the target,
 			List<ValueReference> list = new List<ValueReference> ();
 			list.AddRange (GetMembersSorted (ctx, objectSource, type, proxy, access));
-
-			var names = new ObjectValueNameTracker (ctx);
+			
 			object tdataType = type;
 			
 			foreach (ValueReference val in list) {
@@ -388,7 +387,6 @@ namespace Mono.Debugging.Evaluation
 					}
 					else {
 						ObjectValue oval = val.CreateObjectValue (true);
-						names.FixName (val, oval);
 						values.Add (oval);
 					}
 
@@ -928,10 +926,20 @@ namespace Mono.Debugging.Evaluation
 				string mem = exp.Substring (i, j - i).Trim ();
 				if (mem.Length == 0)
 					return exp;
-
-				ValueReference member = GetMember (ctx, null, GetValueType (ctx, obj), obj, mem);
+				
+				string[] props = mem.Split (new char[] { '.' });
+				ValueReference member = null;
+				object val = obj;
+				
+				for (int k = 0; k < props.Length; k++) {
+					member = GetMember (ctx, null, GetValueType (ctx, val), val, props[k]);
+					if (member == null)
+						break;
+					
+					val = member.Value;
+				}
+				
 				if (member != null) {
-					object val = member.Value;
 					sb.Append (ctx.Evaluator.TargetObjectToString (ctx, val));
 				} else {
 					sb.Append ("{Unknown member '" + mem + "'}");
@@ -1060,33 +1068,6 @@ namespace Mono.Debugging.Evaluation
 		}
 	}
 	
-	class ObjectValueNameTracker
-	{
-		Dictionary<string,KeyValuePair<ObjectValue, ValueReference>> names = new Dictionary<string,KeyValuePair<ObjectValue, ValueReference>> ();
-		EvaluationContext ctx;
-		
-		public ObjectValueNameTracker (EvaluationContext ctx)
-		{
-			this.ctx = ctx;
-		}
-		
-		public void FixName (ValueReference val, ObjectValue oval)
-		{
-			KeyValuePair<ObjectValue, ValueReference> other;
-			if (names.TryGetValue (oval.Name, out other)) {
-				object tn = val.DeclaringType;
-				if (tn != null)
-					oval.Name += " (" + ctx.Adapter.GetDisplayTypeName (ctx, tn) + ")";
-				if (!other.Key.Name.EndsWith (")")) {
-					tn = other.Value.DeclaringType;
-					if (tn != null)
-						other.Key.Name += " (" + ctx.Adapter.GetDisplayTypeName (ctx, tn) + ")";
-				}
-			} else
-				names [oval.Name] = new KeyValuePair<ObjectValue, ValueReference> (oval, val);
-		}
-	}
-	
 	public struct EnumMember
 	{
 		public string Name { get; set; }
