@@ -45,7 +45,8 @@ namespace Mono.TextEditor.Highlighting
 				this.matches = baseMode.Matches;
 				this.prevMarker = baseMode.PrevMarker;
 				this.SemanticRules = new List<SemanticRule> (baseMode.SemanticRules);
-				this.table = baseMode.Table;
+				this.keywordTable = baseMode.keywordTable;
+				this.keywordTableIgnoreCase = baseMode.keywordTableIgnoreCase;
 				this.properties = baseMode.Properties;
 			}
 		}
