@@ -56,6 +56,8 @@ namespace Mono.TextEditor
 		SearchResult SearchBackward (int fromOffset);
 		
 		void Replace (SearchResult result, string pattern);
+		
+		ISearchEngine Clone ();
 	}
 	
 	public abstract class AbstractSearchEngine : ISearchEngine
@@ -121,6 +123,12 @@ namespace Mono.TextEditor
 			return SearchBackward (null, fromOffset);
 		}
 		public abstract void Replace (SearchResult result, string pattern);
+		public virtual ISearchEngine Clone ()
+		{
+			ISearchEngine result = (ISearchEngine)MemberwiseClone ();
+			result.SearchRequest = searchRequest.Clone ();
+			return result;
+		}
 	}
 	
 	public class BasicSearchEngine : AbstractSearchEngine
@@ -150,20 +158,24 @@ namespace Mono.TextEditor
 		
 		public override SearchResult GetMatchAt (int offset)
 		{
-			if ((!string.IsNullOrEmpty (SearchRequest.SearchPattern)) && offset + searchRequest.SearchPattern.Length <= this.textEditorData.Document.Length && compiledPattern.Length > 0) {
+			if (offset < 0)
+				return null;
+			Document doc = this.textEditorData.Document;
+			
+			if ((!string.IsNullOrEmpty (SearchRequest.SearchPattern)) && offset + searchRequest.SearchPattern.Length <= doc.Length && compiledPattern.Length > 0) {
 				if (searchRequest.CaseSensitive) {
-					for (int i = 0; i < compiledPattern.Length; i++) {
-						if (this.textEditorData.Document.GetCharAt (offset + i) != compiledPattern[i]) 
+					for (int i = 0; i < compiledPattern.Length && offset + i < doc.Length; i++) {
+						if (doc.GetCharAt (offset + i) != compiledPattern[i]) 
 							return null;
 					}
 				} else {
-					for (int i = 0; i < compiledPattern.Length; i++) {
-						if (System.Char.ToUpper (this.textEditorData.Document.GetCharAt (offset + i)) != compiledPattern[i]) 
+					for (int i = 0; i < compiledPattern.Length && offset + i < doc.Length; i++) {
+						if (System.Char.ToUpper (doc.GetCharAt (offset + i)) != compiledPattern[i]) 
 							return null;
 					}
 				}
 				if (searchRequest.WholeWordOnly) {
-					if (!this.textEditorData.Document.IsWholeWordAt (offset, compiledPattern.Length))
+					if (!doc.IsWholeWordAt (offset, compiledPattern.Length))
 						return null;
 				}
 				return new SearchResult (offset, compiledPattern.Length, false);
