@@ -32,19 +32,19 @@ using System.Diagnostics;
 using System.IO;
 using System.Reflection;
 using System.Runtime.InteropServices;
+using System.Xml;
 
 namespace MonoDevelop.Core
 {
 	public static class PropertyService
 	{
-		const string CURRENT_PROFILE_VERSION = "2.6";
-		
+		//force the static class to intialize
+		internal static void Initialize ()
+		{
+		}
 		readonly static string FileName = "MonoDevelopProperties.xml";
 		static Properties properties;
 
-		public readonly static bool IsWindows;
-		public readonly static bool IsMac;
-
 		public static Properties GlobalInstance {
 			get { return properties; }
 		}
@@ -57,10 +57,6 @@ namespace MonoDevelop.Core
 			}
 		}
 		
-		public static UserDataLocations Locations {
-			get; private set;
-		}
-		
 		/// <summary>
 		/// Location of data files that are bundled with MonoDevelop itself.
 		/// </summary>
@@ -73,53 +69,20 @@ namespace MonoDevelop.Core
 			}
 		}
 		
-		//From Managed.Windows.Forms/XplatUI
-		static bool IsRunningOnMac ()
-		{
-			IntPtr buf = IntPtr.Zero;
-			try {
-				buf = System.Runtime.InteropServices.Marshal.AllocHGlobal (8192);
-				// This is a hacktastic way of getting sysname from uname ()
-				if (uname (buf) == 0) {
-					string os = System.Runtime.InteropServices.Marshal.PtrToStringAnsi (buf);
-					if (os == "Darwin")
-						return true;
-				}
-			} catch {
-			} finally {
-				if (buf != IntPtr.Zero)
-					System.Runtime.InteropServices.Marshal.FreeHGlobal (buf);
-			}
-			return false;
-		}
-		
-		[System.Runtime.InteropServices.DllImport ("libc")]
-		static extern int uname (IntPtr buf);
-		
 		static PropertyService ()
 		{
 			Counters.PropertyServiceInitialization.BeginTiming ();
 			
-			IsWindows = Path.DirectorySeparatorChar == '\\';
-			IsMac = !IsWindows && IsRunningOnMac ();
-			
-			FilePath testProfileRoot = Environment.GetEnvironmentVariable ("MONODEVELOP_PROFILE");
-			if (!testProfileRoot.IsNullOrEmpty) {
-				Locations = UserDataLocations.ForTest (CURRENT_PROFILE_VERSION, testProfileRoot);
-			} else {
-				Locations = GetLocations (CURRENT_PROFILE_VERSION);
-			}
-			
 			string migrateVersion = null;
-			UserDataLocations migratableProfile = null;
+			UserProfile migratableProfile = null;
 			
-			var prefsPath = Locations.Config.Combine (FileName);
+			var prefsPath = UserProfile.Current.ConfigDir.Combine (FileName);
 			if (!File.Exists (prefsPath)) {
-				if (GetMigratableProfile (testProfileRoot, out migratableProfile, out migrateVersion)) {
-					FilePath migratePrefsPath = migratableProfile.Config.Combine (FileName);
+				if (GetMigratableProfile (out migratableProfile, out migrateVersion)) {
+					FilePath migratePrefsPath = migratableProfile.ConfigDir.Combine (FileName);
 					try {
 						var parentDir = prefsPath.ParentDirectory;
-						//can't use file service until property sevice in initialized
+						//can't use file service until property service is initialized
 						if (!Directory.Exists (parentDir))
 							Directory.CreateDirectory (parentDir);
 						File.Copy (migratePrefsPath, prefsPath);
@@ -149,50 +112,27 @@ namespace MonoDevelop.Core
 			Counters.PropertyServiceInitialization.EndTiming ();
 		}
 		
-		static UserDataLocations GetLocations (string profileVersion)
-		{
-			if (IsWindows)
-				return UserDataLocations.ForWindows (profileVersion);
-			else if (IsMac)
-				return UserDataLocations.ForMac (profileVersion);
-			else
-				return UserDataLocations.ForUnix (profileVersion);
-		}
-		
-		static bool GetMigratableProfile (FilePath testProfileRoot, out UserDataLocations profile, out string version)
+		internal static bool GetMigratableProfile (out UserProfile profile, out string version)
 		{
 			profile = null;
 			version = null;
 			
-			//TODO: check 2.6 when 2.8 is released, etc
-			string[] migratableVersions = { };
-			
-			//test profiles only migrate from test profiles
-			if (!testProfileRoot.IsNullOrEmpty) {
-				for (int i = migratableVersions.Length -1; i >= 0; i--) {
-					var p = UserDataLocations.ForTest (migratableVersions[i], testProfileRoot);
-					if (File.Exists (p.Config.Combine (FileName))) {
-						profile = p;
-						version = migratableVersions[i];
-						return true;
-					}
-				}
-				return false;
-			}
-			
-			//try versioned profiles
-			for (int i = migratableVersions.Length -1; i >= 0; i--) {
-				var p = UserDataLocations.ForTest (migratableVersions[i], testProfileRoot);
-				if (File.Exists (p.Config.Combine (FileName))) {
+			//try versioned profiles from most recent to oldest
+			//skip the last in the array, it's the current profile
+			int userProfileMostRecent = UserProfile.ProfileVersions.Length - 2;
+			for (int i = userProfileMostRecent; i >= 0; i--) {
+				string v = UserProfile.ProfileVersions[i];
+				var p = UserProfile.GetProfile (v);
+				if (File.Exists (p.ConfigDir.Combine (FileName))) {
 					profile = p;
-					version = migratableVersions[i];
+					version = v;
 					return true;
 				}
 			}
 			
 			//try the old unversioned MD <= 2.4 profile
-			var md24 = UserDataLocations.ForMD24 ();
-			if (File.Exists (md24.Config.Combine (FileName))) {
+			var md24 = UserProfile.ForMD24 ();
+			if (File.Exists (md24.ConfigDir.Combine (FileName))) {
 				profile = md24;
 				version = "2.4";
 				return true;
@@ -226,11 +166,16 @@ namespace MonoDevelop.Core
 		public static void SaveProperties ()
 		{
 			Debug.Assert (properties != null);
-			var prefsPath = Locations.Config.Combine (FileName);
+			var prefsPath = UserProfile.Current.ConfigDir.Combine (FileName);
 			FileService.EnsureDirectoryExists (prefsPath.ParentDirectory);
 			properties.Save (prefsPath);
 		}
 		
+		public static bool HasValue (string property)
+		{
+			return properties.HasValue (property);
+		}
+		
 		public static T Get<T> (string property, T defaultValue)
 		{
 			return properties.Get (property, defaultValue);
