@@ -67,10 +67,6 @@ namespace MonoDevelop.Components.Commands
 				SelectionModifierControl = Gdk.ModifierType.ControlMask;
 				SelectionModifierSuper = Gdk.ModifierType.SuperMask;
 			}
-			
-			keymap.KeysChanged += delegate {
-				groupZeroMappings.Clear ();
-			};
 		}
 		
 		static Gdk.Keymap keymap = Gdk.Keymap.Default;
@@ -150,7 +146,8 @@ namespace MonoDevelop.Components.Commands
 		{
 			Gdk.Key key;
 			Gdk.ModifierType modifier;
-			MapRawKeys (raw, out key, out modifier);
+			uint keyVal;
+			Mono.TextEditor.GtkWorkarounds.MapRawKeys (raw, out key, out modifier, out keyVal);
 			
 			//we restore shift modifier if it was consumed for a letter, as accelerators are always 
 			//displayed uppercase, so the shift key must be included in the binding
@@ -300,89 +297,6 @@ namespace MonoDevelop.Components.Commands
 			return mode + "|" + accel;
 		}
 		
-		// NOTE: changes in this should be mirrored in to Mono.TextEditor/Platform.cs
-		public static void MapRawKeys (Gdk.EventKey evt, out Gdk.Key key, out Gdk.ModifierType mod)
-		{
-			mod = evt.State;
-			key = evt.Key;
-			
-			uint keyval;
-			int effectiveGroup, level;
-			Gdk.ModifierType consumedModifiers;
-			keymap.TranslateKeyboardState (evt.HardwareKeycode, evt.State, evt.Group, out keyval, out effectiveGroup,
-			                               out level, out consumedModifiers);
-			
-			key = (Gdk.Key)keyval;
-			mod = evt.State & ~consumedModifiers;
-			
-			if (isX11) {
-				//this is a workaround for a common X mapping issue
-				//where the alt key is mapped to the meta key when the shift modifier is active
-				if (key.Equals (Gdk.Key.Meta_L) || key.Equals (Gdk.Key.Meta_R))
-					key = Gdk.Key.Alt_L;
-			}
-			
-			//HACK: the MAC GTK+ port currently does some horrible, un-GTK-ish key mappings
-			// so we work around them by playing some tricks to remap and decompose modifiers.
-			// We also decompose keys to the root physical key so that the Mac command 
-			// combinations appear as expected, e.g. shift-{ is treated as shift-[.
-			if (isMac && !isX11) {
-				// Mac GTK+ maps the command key to the Mod1 modifier, which usually means alt/
-				// We map this instead to meta, because the Mac GTK+ has mapped the cmd key
-				// to the meta key (yay inconsistency!). IMO super would have been saner.
-				if ((mod & Gdk.ModifierType.Mod1Mask) != 0) {
-					mod ^= Gdk.ModifierType.Mod1Mask;
-					mod |= Gdk.ModifierType.MetaMask;
-				}
-				
-				// If Mod5 is active it *might* mean that opt/alt is active,
-				// so we can unset this and map it back to the normal modifier.
-				if ((mod & Gdk.ModifierType.Mod5Mask) != 0) {
-					mod ^= Gdk.ModifierType.Mod5Mask;
-					mod |= Gdk.ModifierType.Mod1Mask;
-				}
-				
-				// When opt modifier is active, we need to decompose this to make the command appear correct for Mac.
-				// In addition, we can only inspect whether the opt/alt key is pressed by examining
-				// the key's "group", because the Mac GTK+ treats opt as a group modifier and does
-				// not expose it as an actual GDK modifier.
-				if (evt.Group == (byte) 1) {
-					mod |= Gdk.ModifierType.Mod1Mask;
-					key = GetGroupZeroKey (key, evt);
-				}
-			}
-			
-			//fix shift-tab weirdness. There isn't a nice name for untab, so make it shift-tab
-			if (key == Gdk.Key.ISO_Left_Tab) {
-				key = Gdk.Key.Tab;
-				mod |= Gdk.ModifierType.ShiftMask;
-			}
-		}
-		
-		static Dictionary<Gdk.Key,Gdk.Key> groupZeroMappings = new Dictionary<Gdk.Key,Gdk.Key> ();
-		
-		static Gdk.Key GetGroupZeroKey (Gdk.Key mappedKey, Gdk.EventKey evt)
-		{
-			Gdk.Key ret;
-			if (groupZeroMappings.TryGetValue (mappedKey, out ret))
-				return ret;
-			
-			//LookupKey isn't implemented on Mac, so we have to use this workaround
-			uint[] keyvals;
-			Gdk.KeymapKey [] keys;
-			keymap.GetEntriesForKeycode (evt.HardwareKeycode, out keys, out keyvals);
-			
-			//find the key that has the same level (so we preserve shift) but with group 0
-			for (uint i = 0; i < keyvals.Length; i++)
-				if (keyvals[i] == (uint)mappedKey)
-					for (uint j = 0; j < keys.Length; j++)
-						if (keys[j].Group == 0 && keys[j].Level == keys[i].Level)
-							return groupZeroMappings[mappedKey] = ret = (Gdk.Key)keyvals[j];
-			
-			//failed, but avoid looking it up again
-			return groupZeroMappings[mappedKey] = mappedKey;
-		}
-		
 		static string ModifierToPartialAccel (Gdk.ModifierType mod, Gdk.Key key, out bool keyIsModifier)
 		{
 			string label = String.Empty;
