@@ -35,14 +35,14 @@ using MonoDevelop.Core;
 using Mono.Addins;
 using MonoDevelop.Projects.Text;
 using Gtk;
+using MonoDevelop.Ide.Gui;
 
 #pragma warning disable 612
 
 namespace MonoDevelop.Ide.Gui.Dialogs
 {
-	public class FileSelectorDialog: FileSelector
+	class FileSelectorDialog: FileSelector
 	{
-		Hashtable filterPairs;
 		int selectOption;
 		int firstEncIndex;
 		
@@ -61,50 +61,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		{
 			LocalOnly = true;
 			
-			ArrayList filters = new ArrayList ();
-			filters.AddRange (AddinManager.GetExtensionObjects ("/MonoDevelop/Ide/ProjectFileFilters"));
-			try
-			{
-				filters.AddRange (AddinManager.GetExtensionObjects ("/MonoDevelop/Ide/FileFilters"));
-			}
-			catch
-			{
-				//nothing there..	
-			}
-			
-			filterPairs = new Hashtable ();
-			foreach (string filterStr in filters)
-			{
-				string[] parts = filterStr.Split ('|');
-				Gtk.FileFilter filter = new Gtk.FileFilter ();
-				filter.Name = parts[0];
-				filterPairs[parts[0]] = parts[1];
-				foreach (string ext in parts[1].Split (';'))
-				{
-					filter.AddPattern (ext);
-				}
-				AddFilter (filter);
-			}
-			
-			//Add All Files
-			Gtk.FileFilter allFilter = new Gtk.FileFilter ();
-			allFilter.Name = GettextCatalog.GetString ("All Files");
-			allFilter.AddPattern ("*");
-			filterPairs[GettextCatalog.GetString ("All Files")] = ("*");
-			AddFilter (allFilter);
-			
-			// Load last used filter
-			string lastPattern = (string)PropertyService.Get ("Monodevelop.FileSelector.LastPattern", "*");
-			foreach (FileFilter filter in this.Filters)
-			{
-				string pattern = filterPairs[filter.Name] as string;
-				if (! String.IsNullOrEmpty (pattern) && pattern == lastPattern)
-				{
-					this.Filter = filter;
-					break;
-				}
-			}
-			
 			// Add the text encoding selector
 			Table table = new Table (2, 2, false);
 			table.RowSpacing = 6;
@@ -148,7 +104,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				ShowEncodingSelector = false;
 				
 			if (action != Gtk.FileChooserAction.Open)
-				ShowViewerSelector = false;
+				closeWorkspaceCheck.Visible = ShowViewerSelector = false;
 		}
 		
 		public string Encoding {
@@ -239,10 +195,8 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		void EncodingChanged (object s, EventArgs args)
 		{
 			if (encodingMenu.History == selectOption) {
-				SelectEncodingsDialog dlg = new SelectEncodingsDialog ();
-				dlg.TransientFor = this;
-				dlg.Run ();
-				dlg.Destroy ();
+				var dlg = new SelectEncodingsDialog ();
+				MessageService.ShowCustomDialog (dlg, this);
 				FillEncodings ();
 			}
 		}
@@ -259,8 +213,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				viewerSelector.AppendText (GettextCatalog.GetString ("Solution Workbench"));
 				currentViewers.Add (null);
 			}
-			FileViewer[] vs = IdeApp.Workbench.GetFileViewers (Filename);
-			foreach (FileViewer vw in vs) {
+			foreach (FileViewer vw in DisplayBindingService.GetFileViewers (Filename, null)) {
 				if (!vw.IsExternal) {
 					viewerSelector.AppendText (vw.Title);
 					currentViewers.Add (vw);
@@ -314,16 +267,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			UpdateExtraWidgets ();
 			FillViewers ();
 		}
-		
-		protected override void OnDestroyed ()
-		{
-			// Save active filter
-			string pattern = filterPairs[this.Filter.Name] as string;
-			if (pattern != null)
-				PropertyService.Set ("Monodevelop.FileSelector.LastPattern", pattern);
-
-			base.OnDestroyed ();
-		}
 	}
 }
 
