@@ -27,7 +27,9 @@
 //
 
 using System;
+using System.Linq;
 using System.Collections;
+using System.Collections.Generic;
 using System.Diagnostics;
 using System.IO;
 using System.Text;
@@ -178,13 +180,10 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			Project sourceProject;
 			System.Collections.Generic.IEnumerable<ProjectFile> groupedChildren = null;
 			
-			bool ask;
-			
 			if (dataObject is ProjectFolder) {
 				source = ((ProjectFolder) dataObject).Path;
 				sourceProject = ((ProjectFolder) dataObject).Project;
 				what = Path.GetFileName (source);
-				ask = true;
 			}
 			else if (dataObject is ProjectFile) {
 				ProjectFile file = (ProjectFile) dataObject;
@@ -196,7 +195,6 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				}
 				groupedChildren = file.DependentChildren;
 				what = null;
-				ask = false;
 			}
 			else if (dataObject is Gtk.SelectionData) {
 				SelectionData data = (SelectionData) dataObject;
@@ -223,15 +221,15 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			targetPath = targetPath.Combine (source.FileName);
 			// If copying to the same directory, make a copy with a different name
 			if (targetPath == source)
-				targetPath = GetTargetCopyName (targetPath, dataObject is ProjectFolder);
+				targetPath = ProjectOperations.GetTargetCopyName (targetPath, dataObject is ProjectFolder);
 			
-			if (ask) {
+			if (dataObject is ProjectFolder) {
 				string q;
 				if (operation == DragOperation.Move) {
 					if (targetPath.ParentDirectory == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, targetPath.FileName);
+						q = GettextCatalog.GetString ("Do you really want to move the folder '{0}' to the folder '{1}'?", what, targetPath.ParentDirectory.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Move))
 						return;
 				}
@@ -239,11 +237,14 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 					if (targetPath.ParentDirectory == targetProject.BaseDirectory)
 						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the root folder of project '{1}'?", what, targetProject.Name);
 					else
-						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, targetPath.FileName);
+						q = GettextCatalog.GetString ("Do you really want to copy the folder '{0}' to the folder '{1}'?", what, targetPath.ParentDirectory.FileName);
 					if (!MessageService.Confirm (q, AlertButton.Copy))
 						return;
 				}
-
+			} else if (dataObject is ProjectFile) {
+				if (File.Exists (targetPath))
+					if (!MessageService.Confirm (GettextCatalog.GetString ("The file '{0}' already exists. Do you want to overwrite it?", targetPath.FileName), AlertButton.OverwriteFile))
+						return;
 			}
 			
 			ArrayList filesToSave = new ArrayList ();
@@ -302,54 +303,15 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			
 			using (IProgressMonitor monitor = IdeApp.Workbench.ProgressMonitors.GetStatusProgressMonitor (GettextCatalog.GetString("Copying files..."), MonoDevelop.Ide.Gui.Stock.CopyIcon, true))
 			{
+				// If we drag and drop a node in the treeview corresponding to a directory, do not move
+				// the entire directory. We should only move the files which exist in the project. Otherwise
+				// we will need a lot of hacks all over the code to prevent us from incorrectly moving version
+				// control related files such as .svn directories
 				bool move = operation == DragOperation.Move;
-				IdeApp.ProjectOperations.TransferFiles (monitor, sourceProject, source, targetProject, targetPath, move, false);
+				IdeApp.ProjectOperations.TransferFiles (monitor, sourceProject, source, targetProject, targetPath, move, true);
 			}
 		}
 		
-		internal static FilePath GetTargetCopyName (FilePath path, bool isFolder)
-		{
-			int n=1;
-			// First of all try to find an existing copy tag
-			string fn = path.FileNameWithoutExtension;
-			for (int i=1; i<100; i++) {
-				string copyTag = GetCopyTag (i); 
-				if (fn.EndsWith (copyTag)) {
-					string newfn = fn.Substring (0, fn.Length - copyTag.Length);
-					if (newfn.Trim ().Length > 0) {
-						n = i + 1;
-						path = path.ParentDirectory.Combine (newfn + path.Extension);
-						break;
-					}
-				}
-			}
-			FilePath basePath = path;
-			while ((!isFolder && File.Exists (path)) || (isFolder && Directory.Exists (path))) {
-				string copyTag = GetCopyTag (n);
-				path = basePath.ParentDirectory.Combine (basePath.FileNameWithoutExtension + copyTag + basePath.Extension);
-				n++;
-			}
-			return path;
-		}
-		
-		static string GetCopyTag (int n)
-		{
-			string sc;
-			switch (n) {
-				case 1: sc = GettextCatalog.GetString ("copy"); break;
-				case 2: sc = GettextCatalog.GetString ("another copy"); break;
-				case 3: sc = GettextCatalog.GetString ("3rd copy"); break;
-				case 4: sc = GettextCatalog.GetString ("4th copy"); break;
-				case 5: sc = GettextCatalog.GetString ("5th copy"); break;
-				case 6: sc = GettextCatalog.GetString ("6th copy"); break;
-				case 7: sc = GettextCatalog.GetString ("7th copy"); break;
-				case 8: sc = GettextCatalog.GetString ("8th copy"); break;
-				case 9: sc = GettextCatalog.GetString ("9th copy"); break;
-				default: sc = GettextCatalog.GetString ("copy {0}"); break;
-			}
-			return " (" + string.Format (sc, n) + ")";
-		}
-		
 		[CommandHandler (ProjectCommands.AddFiles)]
 		public void AddFilesToProject()
 		{
@@ -372,14 +334,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			ProjectFolder folder = CurrentNode.GetParentDataItem (typeof(ProjectFolder), true) as ProjectFolder;
 			FilePath baseDirectory = folder != null ? folder.Path : project.BaseDirectory;
 			
-			var addedFiles = IdeApp.ProjectOperations.AddFilesToProject (project, files, baseDirectory);
-			
-			//override the build action of the added files if needed
-			if (!string.IsNullOrEmpty (overrideAction)) {
-				foreach (var pf in addedFiles)
-					if (pf != null)
-						pf.BuildAction = overrideAction;
-			}
+			IdeApp.ProjectOperations.AddFilesToProject (project, files, baseDirectory, overrideAction);
 			
 			IdeApp.ProjectOperations.Save (project);
 		}
@@ -391,6 +346,8 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			IdeApp.ProjectOperations.CreateProjectFile (project, GetFolderPath (CurrentNode.DataItem));
 			IdeApp.ProjectOperations.Save (project);
 			CurrentNode.Expanded = true;
+			if (IdeApp.Workbench.ActiveDocument != null)
+				IdeApp.Workbench.ActiveDocument.Window.SelectWindow ();
 		}
 		
 		void OnFileInserted (ITreeNavigator nav)
@@ -398,6 +355,86 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			nav.Selected = true;
 			Tree.StartLabelEdit ();
 		}
+
+		///<summary>Imports files and folders from a target folder into the current folder</summary>
+		[CommandHandler (ProjectCommands.AddFilesFromFolder)]
+		public void AddFilesFromFolder ()
+		{
+			var project = (Project) CurrentNode.GetParentDataItem (typeof(Project), true);
+			var targetRoot = ((FilePath) GetFolderPath (CurrentNode.DataItem)).CanonicalPath;
+			
+			var ofdlg = new SelectFolderDialog (GettextCatalog.GetString ("Import From Folder")) {
+				CurrentFolder = targetRoot
+			};
+			if(!ofdlg.Run ())
+				return;
+			
+			var srcRoot = ofdlg.SelectedFile.CanonicalPath;
+			var foundFiles = Directory.GetFiles (srcRoot, "*", SearchOption.AllDirectories);
+			
+			var impdlg = new IncludeNewFilesDialog (GettextCatalog.GetString ("Select files to add from {0}", srcRoot.FileName), srcRoot);
+			impdlg.AddFiles (foundFiles);
+			if (MessageService.ShowCustomDialog (impdlg) != (int) ResponseType.Ok)
+				return;
+				
+			var srcFiles = impdlg.SelectedFiles;
+			var targetFiles = srcFiles.Select (f => targetRoot.Combine (f.ToRelative (srcRoot)));
+
+			var added = IdeApp.ProjectOperations.AddFilesToProject (project, srcFiles.ToArray (), targetFiles.ToArray (), null).Any ();
+			if (added)
+				IdeApp.ProjectOperations.Save (project);
+		}
+		
+		///<summary>Adds an existing folder to the current folder</summary>
+		[CommandHandler (ProjectCommands.AddExistingFolder)]
+		public void AddExistingFolder ()
+		{
+			var project = (Project) CurrentNode.GetParentDataItem (typeof(Project), true);
+			var selectedFolder = ((FilePath) GetFolderPath (CurrentNode.DataItem)).CanonicalPath;
+			
+			var ofdlg = new SelectFolderDialog (GettextCatalog.GetString ("Add Existing Folder")) {
+				CurrentFolder = selectedFolder
+			};
+			if(!ofdlg.Run ())
+				return;
+			
+			var srcRoot = ofdlg.SelectedFile.CanonicalPath;
+			var targetRoot = selectedFolder.Combine (srcRoot.FileName);
+
+			bool changedProject = false;
+
+			if (File.Exists (targetRoot)) {
+				MessageService.ShowWarning (GettextCatalog.GetString (
+					"There is already a file with the name '{0}' in the target directory", srcRoot.FileName));
+				return;
+			}
+
+			var existingPf = project.Files.GetFileWithVirtualPath (targetRoot.ToRelative (project.BaseDirectory));
+			if (existingPf != null) {
+				if (existingPf.Subtype != Subtype.Directory) {
+					MessageService.ShowWarning (GettextCatalog.GetString (
+						"There is already a link with the name '{0}' in the target directory", srcRoot.FileName));
+					return;
+				} else {
+					project.Files.Add (new ProjectFile (targetRoot) { Subtype = Subtype.Directory });
+					changedProject = true;
+				}
+			}
+
+			var foundFiles = Directory.GetFiles (srcRoot, "*", SearchOption.AllDirectories);
+			
+			var impdlg = new IncludeNewFilesDialog (GettextCatalog.GetString ("Select files to add from {0}", srcRoot.FileName), srcRoot.ParentDirectory);
+			impdlg.AddFiles (foundFiles);
+			if (MessageService.ShowCustomDialog (impdlg) == (int) ResponseType.Ok) {
+				var srcFiles = impdlg.SelectedFiles;
+				var targetFiles = srcFiles.Select (f => targetRoot.Combine (f.ToRelative (srcRoot)));
+				if (IdeApp.ProjectOperations.AddFilesToProject (project, srcFiles.ToArray (), targetFiles.ToArray (), null).Any ())
+					changedProject = true;
+			}
+			
+			if (changedProject)
+				IdeApp.ProjectOperations.Save (project);
+		}
 		
 		[CommandHandler (ProjectCommands.NewFolder)]
 		public void AddNewFolder ()
