@@ -44,6 +44,7 @@ using MonoDevelop.Components.DockToolbars;
 using Gtk;
 using MonoDevelop.Components;
 using MonoDevelop.Ide.Extensions;
+using Mono.TextEditor;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -78,8 +79,6 @@ namespace MonoDevelop.Ide.Gui
 		bool fullscreen;
 		Rectangle normalBounds = new Rectangle(0, 0, 640, 480);
 		
-		internal static GType gtype;
-		
 		Gtk.Container rootWidget;
 		DockToolbarFrame toolbarFrame;
 		DockFrame dock;
@@ -211,15 +210,7 @@ namespace MonoDevelop.Ide.Gui
 
 			DeleteEvent += new Gtk.DeleteEventHandler (OnClosing);
 			
-			if (Gtk.IconTheme.Default.HasIcon ("monodevelop")) 
-				Gtk.Window.DefaultIconName = "monodevelop";
-			else
-				this.IconList = new Gdk.Pixbuf[] {
-					ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Menu),
-					ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Button),
-					ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Dnd),
-					ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Dialog)
-				};
+			SetAppIcons ();
 
 			//this.WindowPosition = Gtk.WindowPosition.None;
 
@@ -228,6 +219,37 @@ namespace MonoDevelop.Ide.Gui
 			
 			IdeApp.CommandService.SetRootWindow (this);
 		}
+		
+		void SetAppIcons ()
+		{
+			//first try to get the icon from the GTK icon theme
+			var appIconName = BrandingService.GetString ("ApplicationIconId")
+				?? BrandingService.ApplicationName.ToLower ();
+			if (Gtk.IconTheme.Default.HasIcon (appIconName)) {
+				Gtk.Window.DefaultIconName = appIconName;
+				return;
+			}
+			
+			//branded icons
+			var iconsEl = BrandingService.GetElement ("ApplicationIcons");
+			if (iconsEl != null) {
+				try {
+					this.IconList = iconsEl.Elements ("Icon")
+						.Select (el => new Gdk.Pixbuf (BrandingService.GetFile ((string)el))).ToArray ();
+					return;
+				} catch (Exception ex) {
+					LoggingService.LogError ("Could not load app icons", ex);
+				}
+			}
+			
+			//built-ins
+			this.IconList = new Gdk.Pixbuf[] {
+				ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Menu),
+				ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Button),
+				ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Dnd),
+				ImageService.GetPixbuf (MonoDevelop.Ide.Gui.Stock.MonoDevelop, Gtk.IconSize.Dialog)
+			};
+		}
 
 		void onDragDataRec (object o, Gtk.DragDataReceivedArgs args)
 		{
@@ -807,16 +829,7 @@ namespace MonoDevelop.Ide.Gui
 					ToggleFullViewMode ();
 			};
 			
-			this.tabControl.PopupMenu += delegate {
-				ShowPopup ();
-			};
-			this.tabControl.ButtonReleaseEvent += delegate (object sender, Gtk.ButtonReleaseEventArgs e) {
-				int tab = tabControl.FindTabAtPosition (e.Event.XRoot, e.Event.YRoot);
-				if (tab < 0)
-					return;
-				if (e.Event.Button == 3)
-					ShowPopup ();
-			};
+			this.tabControl.DoPopupMenu = ShowPopup;
 			
 			tabControl.TabsReordered += new TabsReorderedHandler (OnTabsReordered);
 
@@ -939,11 +952,10 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
-		void ShowPopup ()
+		void ShowPopup (int tabIndex, Gdk.EventButton evt)
 		{
-			Gtk.Menu contextMenu = IdeApp.CommandService.CreateMenu ("/MonoDevelop/Ide/ContextMenu/DocumentTab");
-			if (contextMenu != null)
-				contextMenu.Popup ();
+			this.tabControl.Page = tabIndex;
+			IdeApp.CommandService.ShowContextMenu (this.tabControl, evt, "/MonoDevelop/Ide/ContextMenu/DocumentTab");
 		}
 		
 		void OnTabsReordered (Widget widget, int oldPlacement, int newPlacement)
@@ -1384,6 +1396,28 @@ namespace MonoDevelop.Ide.Gui
 				rect.Height -= CurrentPageWidget.Allocation.Height;
 			yield return rect;
 		}
+		
+		public Action<int,Gdk.EventButton> DoPopupMenu { get; set; }
+		
+		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
+		{
+			if (DoPopupMenu != null && evnt.TriggersContextMenu ()) {
+				int tab = FindTabAtPosition (evnt.XRoot, evnt.YRoot);
+				if (tab >= 0) {
+					DoPopupMenu (tab, evnt);
+					return true;
+				}
+			}
+			return base.OnButtonPressEvent (evnt);
+		}
+		
+		protected override bool OnPopupMenu ()
+		{
+			if (DoPopupMenu != null) {
+				DoPopupMenu (this.Page, null);
+				return true;
+			}
+			return base.OnPopupMenu ();
+		}
 	}
-}
-
+}
\ No newline at end of file
